/*
  Author:  Jonathan Bondzie, Dr. Chuck Yeung
           Lennard Gas Simulation Source

	   Copyright (C) 1999  Penn State University

	   This program is free software; you can redistribute it and/or
	   modify it under the terms of the GNU General Public License
	   as published by the Free Software Foundation;
	   
	   This program is distributed in the hope that it will be useful,
	   but WITHOUT ANY WARRANTY; without even the implied warranty of
	   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	   GNU General Public License for more details.
	   
	   You should have received a copy of the GNU General Public License
	   along with this program; if not, write to the Free Software
	   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
	   02111-1307, USA.

	   email jab341@psu.edu
*/

#include "motifR.h"
#include "Headers.h"
#define tempconstant 862.507157 /* conversion factor for the temperature */
#define energyconstant 1.19E-20 /* conversion factor for the energies */


/* prototype for creating the colormap */

void CreateColormap(void);


/**************************************************************************/
/*	               drawing routines					  */
/**************************************************************************/

#define  UPDATE_COUNTER	10
#define	 FULLCIRCLE	23040
		/* argument for draw arc routines 23040 = 360*64
		 *	64 increments per degree
		 */


/* these Variables needed for DrawCircles and refreshDisplayAndScales */

static int diameter;            /* diameter of the particles in pixels */
static Dimension width;		/* width of drawing area */
static Dimension height;        /* height of drawing area */
static float pixel_scale;       /* scaling from x,y to pixels */
static int  ydisplace;		/* displaces the y positions so
				 * that the bottom of the box
				 * is always on screen -- this is only
				 * nonzero when the whole y region does
				 * not fit on the screen due to the
				 * large aspect ratio
				 */

/* window variables */

static  int	 window;
static  Display* display;
static  Window	 pixmap;	/* working pixmap */	 
static	int	 depth;		/* depth of window -- not used */
static	unsigned long fore, back;/*fg and bg colors */
static  int max_colors = 0;      /* if XAllocColors returns a false
                                    presumably there isn't enough memory
                                    to allocate the colors. Therefore the
                                    the brightest shade of red will be 
                                    at the indes xcolor[max_colors]*/ 

static XColor xcolor[256];
void DrawCircles(Variables* ptr)
{

  int	k;		       /* loop index */
  int	i1, j1;		       /* relative pixel positions */
  static float temp_interval=0;/* the update time interval for
				  the temperature.             */
  static float avg_checker;    /* we need a TRUE counter for however many times
				  the value of the avg_kinetic changed */
  static float ptime =0;       /* plot interval */
	
  float tke =0,                /* temporary kinetic,potential and total energies */
        tpe =0,
        tte =0;
  float instant_temp       = 0;
  float instant_velocity   = 0; 
  int color_index          = 0;          /* color of the particles based on its velocity */
  static float avg_kinetic = 0;
  static unsigned int avg_counter=0;     /* counts the number of data points collectd */
  Widgets* wptr;
  
  wptr = ReturnClassAddress();
  refreshDisplayAndScales(ptr);

  /* clear the working pixmap  by setting the foreground color
  *  equal to the background color and drawing a filled rectangle
  *  into entire pixmap
  */

  XSetForeground(display, (*wptr).dgr_gc,back);
  XFillRectangle(display, pixmap, (*wptr).dgr_gc,0,0,width,height); 

  /*create the  particles */

  for( k = 0; k < (*ptr).nparticles; k++)
  {
	instant_velocity = sqrt((*ptr).vx[k] * (*ptr).vx[k]) + ((*ptr).vy[k] * (*ptr).vy[k] );
        color_index =  (int)(max_colors - (instant_velocity/(16.0/max_colors)));
        color_index =  (int)floor(color_index + .5);
        if(color_index < 0)
	  color_index= 0;
	if(color_index > max_colors ) 
          color_index = max_colors;
	XSetForeground(display, (*wptr).dgr_gc,xcolor[color_index].pixel);
	i1 = (int)( pixel_scale * ( (*ptr).x[k] - 0.5 )) ;
	j1 = (int)( pixel_scale * ( (*ptr).ly - (*ptr).y[k] - 0.5 ) - ydisplace); 
	XFillArc(display, pixmap, (*wptr).dgr_gc,i1, j1, diameter, diameter, 0, FULLCIRCLE);
  }

  /* draw boxes to show blocked regions for shaking walls */

  XSetForeground(display, (*wptr).dgr_gc, 14L);
  i1 = (int)( pixel_scale * (*ptr).ly) - ydisplace;
  j1 = (int)((*ptr).ly*pixel_scale);
  XFillRectangle(display, pixmap, (*wptr).dgr_gc, 0, i1, width, j1);

  XSetForeground(display, (*wptr).dgr_gc, 14L);
  j1 = (int) (-1*ydisplace);
  if( j1 < 0 ) j1 = 0;
  XFillRectangle(display, pixmap, (*wptr).dgr_gc, 0, 0, width, j1);

  /* copy the working pixmap to the display window */

  XCopyArea(display, pixmap, window, (*wptr).dgr_gc, 0, 0, width, height,0, 0);

  pasteFloatToWidget((float)(floor(10*(*ptr).t)/10), 4, (*wptr).dgr_time_text,(XmTextPosition) 0);
		
	
      
  /**************************************************************************
   *
   *	                 update the energies                                    
   * 
   **************************************************************************/


   /* avg_kinetic is the total system temperature at some time t */

   avg_checker = avg_kinetic;
   avg_kinetic += (double) ((*ptr).kinetic_energy * tempconstant)/((*ptr).nparticles /*blennard*/);
   instant_temp = ((*ptr).kinetic_energy * tempconstant)/((*ptr).nparticles /*blennard*/); 
     
   if((*ptr).reset == TRUE)
   {
     avg_counter= 0;
     avg_kinetic = 0;
     (*ptr).total_pressure =0;
     (*ptr).fv_lx = 0; 
     (*ptr).fv_ly = 0;
     temp_interval=0;
     ptime=0;
     (*ptr).reset = FALSE;
   }
   
   if(avg_checker != avg_kinetic)
   {
     avg_counter++;
     /* since the drawing routine gets called when there are no 
	events going on, avg_kinetic gets recalculated, i.e. if we
	just count however many times avg_kinetic gets recalculated
	it's far more than, it is TRUE. It's very unlikely that the 
	value of avg_kinetic could stay the same with particles moving 
	so the counter only gets incremented when the value of avg_kinetic
	gets changed.                                                     */

     /* note: avg_checker could produce some unexpected results
	if it loops around. make sure this doen't happen and if 
	by any chance it does I hope I will not have to fix it */
   }

   if( (*ptr).t - ptime >= 5 )
   {
     Function(ptr);
     ptime = (*ptr).t;
   }
  
   if((*ptr).t - temp_interval >= (*ptr).ftime_interval)
   {	
     (*ptr).temp = (avg_kinetic / avg_counter);
     temp_interval = (*ptr).t;
   }
     
   tke = ((*ptr).kinetic_energy/(*ptr).nparticles)*(energyconstant /*blennard*/);
   tte = ((*ptr).total_energy / (*ptr).nparticles)*(energyconstant /*blennard*/);
   tpe = ((*ptr).potential_energy / (*ptr).nparticles)*(energyconstant /*blennard*/);

   SetTextString((*wptr).dgr_kinetic_window,(double)tke);
   SetTextString((*wptr).dgr_total_energy_window,(double)tte);
   SetTextString((*wptr).dgr_potential_window,(double)tpe);
   
   /* decision to display either the integrated physical quantities or the instant */

   switch((*ptr).display_integral)
   {
     case 1:
     { 
       SetTextString((*wptr).dgr_itemp,(double)(*ptr).temp);
       SetTextString((*wptr).dgr_ipressure,(double)(*ptr).total_pressure);
       break;
     }

    case 0:
     {
       SetTextString((*wptr).dgr_itemp,(double)instant_temp);
       SetTextString((*wptr).dgr_ipressure,(double)(*ptr).instant_pressure);
     }
   }

   /* flush and update everything */

   XFlush(display);
   XmUpdateDisplay((*wptr).dgr_mainwindow); 
}


/********************************************************************
 *	    refreshDisplayAndScales
 *
 *	Initializes the drawing area window if first time also
 * refreshes the main window making sure that all resizes etc. are
 *  taken care of by recalculating the diameter,  pixel_scale and pixel_scale
 *  scales in terms of present widht and height of window
 *
 *  This was separated so that it can be called during resize commands
 *
 */

void refreshDisplayAndScales(Variables* ptr)
{
 
  static int firsttime = TRUE;
  static Dimension initialwidth=width;
  static float xscale = width/(*ptr).lx; /* width to lx ratio */
  static Widgets* wptr;

  /* initializes the drawing window */

  if( firsttime == TRUE )
  {

   wptr = ReturnClassAddress();

   /* get display and window data if first time thru */

    display = XtDisplay((*wptr).dgr_drawing_area);
    window = XtWindow((*wptr).dgr_drawing_area);
   
    /* create the colormap */
    CreateColormap();

    /* get foreground and background colors if first time thru */
    
    XtVaGetValues((*wptr).dgr_drawing_area, XmNbackground, &back,
		  XmNforeground, &fore, XmNdepth, &depth, NULL);
    
    /* create the working pixmap with correct width and height 
     * The pixmap has the same graphical context as the window.
     */
    
    XtVaGetValues((*wptr).dgr_drawing_area, XmNwidth, &width, XmNheight,
		  &height, NULL);
    pixmap = XCreatePixmap(display, window, width,
			   height, depth);

    
    firsttime = FALSE;
       
    /* diameter of circles in pixels */
    
    diameter = (int)( width/(*ptr).lx + 0.5 );
    if ( diameter <= 1 )
      diameter = 2;
    initialwidth = width;
    xscale = width/(*ptr).lx;

   }

  /* get width and height and scale in pixels */
  
  XtVaGetValues((*wptr).dgr_drawing_area, XmNwidth, &width, XmNheight,
		&height, NULL);
 
  RescaleGraphics(initialwidth,width,&pixel_scale,&diameter,(*ptr).lx); 
   
  /* ydisplace = amount y displaced to keep bottom of box in picture */
  
  if( (int)height < pixel_scale * (*ptr).ly)
    ydisplace = (int) ( (pixel_scale* (*ptr).ly)-(int)height );
  else ydisplace = 0;
  

 /* free old pixmap and creates new one in case a resize has occured */
  
  XFreePixmap(display, pixmap);
  pixmap = XCreatePixmap(display, window, width, height, depth);
  
  XmUpdateDisplay((*wptr).dgr_mainwindow);
  
}


/* this routine creates the colormap for the particles */

void CreateColormap(void)
{
  Colormap colormap;

  /* create a colormap */

  colormap = DefaultColormap(display,(DefaultScreen(display)));
   for(int i=0;i<255;i++)
   {
     xcolor[i].flags = DoRed|DoBlue|DoGreen;
     xcolor[i].red   = 65535;
     xcolor[i].blue  = i*256+1;
     xcolor[i].green = i*256+1;
     if(XAllocColor(display,colormap,&xcolor[i]))
       max_colors = i;
     else
      break;
   } 
}
